<?php

    // import time limit in seconds, once reached the process will stop importing any new data and finish cleanly, ideally set it long enough to do a full import
    $process_time_limit = 600;

    // if we're running a process then abort this to avoid infinite loops
    if (defined("process_running")){exit;}else{define("process_running",true);} // move this into main.php cron ?
    $cron=1;
    require_once("../../../../scripts/main.php");

    // make sure api does not run if cms already locked
    if (isLocked('api')) exit;

    require('api.php');

    // check for security token
    sleep(1); // wait a bit to make sure token and process log files are created
    if (isset($_GET['stoken']) && $token_details=check_security_token($_GET['stoken'],"",true)){

       // defaults for args
        $rvi_import_items = 1;
        $rvi_import_start_item=1;
        $rvi_import_limit = 0;
        $rvi_import_images = 0;
        // get args from token_details
        if (!empty($token_details['custom_string'])){
            $token_args=explode("&",$token_details['custom_string']);
            if (!empty($token_args)){
                foreach ($token_args as $arg_str){
                    $arg_pieces=explode("=",$arg_str);
                    $arg[$arg_pieces[0]]=$arg_pieces[1];
                }

                if (isset($arg['import_items'])) $rvi_import_items = $arg['import_items'];
                if (isset($arg['import_start_item'])) $rvi_import_start_item = $arg['import_start_item'];
                if (isset($arg['import_limit'])) $rvi_import_limit = $arg['import_limit'];
                if (isset($arg['import_images'])) $rvi_import_images = $arg['import_images'];

            }

        }

        update_api_import_log($api_details['id'],"Process API ".$api_details['id']." started ".$rvi_import_items."_".$rvi_import_start_item."_".$rvi_import_limit."_".$rvi_import_images."..");
    }else{
        update_api_import_log($api_details['id'],"Warning: Process API ".$api_details['id']." aborted - no security token");
        exit;
    }

    // importing the data
    $api_data_fn=$api_data_dir.'/data.php';
    $api_temp_fn=$api_data_dir.'/file.temp'; // used to import image files
    $api_item_count=0;
    $updated_item_count=0;
    $overwrite_item_count=0;
    $omit_item_count=0;
    $import_img_count=0;
    $import_img_dl_fail=0;

    // check headers and include them in image checksum for determining changes to image file or set to false to just use url in api data as checksum
    // checking headers content-length also allows for skipping import on very large images so although slower recommended to use this
    $check_img_headers=true;

    if (!file_exists($api_data_fn)){
        update_api_import_log($api_details['id'],"Error: API ".$api_details['id']." import data failed - data file not found");
        exit;
    }

    // import csv data
    if (($handle = fopen($api_data_fn,'r')) !== FALSE) {

        // lock api and lock products while processing
        $unlock_key=lock_cms();

        if (fgets($handle, 4) !== "\xef\xbb\xbf") //Skip BOM if present
            rewind($handle); //Or rewind pointer to start of file


        // each category name in the import data can be tied to multiple category ids
        $api_cat_name_to_cms_cat_ids=array(
        'mountain bikes' => array('1616908290405'),
        'mountain' => array('1616908290405'),
        'mtb ds' => array('1616908290405','1618407096485','1616908433749'),
        'mtb fs' => array('1616908290405','1618407073318','1616908433749'),
        'e-cycling' => array('1616908411384','1616908455118','1618407073318','1618407096485','1616908433749'),
        );

        // csvcol array has column headings as the key and the value is the column number
        $csv_line_longest_length=6000; // Must be greater than the longest line (in characters) to be found in the CSV file
        $csvcol=array();
        // import data loop
        while (($api_data = fgetcsv($handle, $csv_line_longest_length, ',')) !== FALSE) {

            // skip lines until finds heading line
            if (empty($csvcol)){

                // the heading (lowercased) we use for item id, must exist to import
                $item_id_csv_heading = 'id';

                //$csv_headings=$api_data;
                // import_headings array should include the lowercase names of the headings of the column data you want to import
                //$import_headings=array('upc_code','sku','retail_price','short_description','long_description','image','brand','model','size','color','frame','fork','head_set','stem','handle_bars','tape_grips','brakes','tyres','front_derailleur','rear_derailleur','shift_levers','cassette','crankset','bottom_bracket','pedals','wheels','front_tyre','rear_tyre','saddle','seat_post');
                foreach($api_data as $key => $value){
                    $csv_heading=strtolower(trim($value));
                    // use strpos on first col heading to avoid issues with bom data at start of file
                    // if (!isset($csvcol['upc_code']) && strpos($csv_heading,'upc_code')!==false){$csvcol['upc_code']=$key;continue;}

                    if (!empty($csv_heading) && !isset($csvcol[$csv_heading])) $csvcol[$csv_heading]=$key;

                    // if the heading for product id is not found clear cvscol and we try next line
                    if (!isset($csvcol[$item_id_csv_heading])) $csvcol=array();
                    /*
                    if (in_array($csv_heading,$import_headings) && !isset($csvcol[$csv_heading])){
                        $csvcol[$csv_heading]=$key;
                        continue;
                    }*/

                }

                continue;
            }

            if (!isset($csvcol[$item_id_csv_heading])) break; // if no heading set for product id then break

            if (empty($api_data[$csvcol[$item_id_csv_heading]])) continue; // if line has no or blank product id then continue to next line

            set_time_limit(30); // restart time limit per item import to ensure enough time for import data, categories, image downloads and any connenction timeouts in one product

            $api_item_count++;

            // skip if less than start item
            if ($api_item_count<$rvi_import_start_item) continue;

            $update_product=0;
            $api_data_checksum=crc32(serialize($api_data));

            $product = new Product();
            if (!$product->setId($api_data[$csvcol[$item_id_csv_heading]])) continue; // set id, cms always stores item id lowercase, invalid characters we skip to next item

            if ($product->getData()){
                if (empty($product->checksums[$api_details['id']]) || $product->checksums[$api_details['id']]!=$api_data_checksum || $rvi_import_items>1){
                   $update_product = 1;
                   $overwrite_item_count++;
                }
            }
            else
                $update_product = 1;

            if ($update_product){

                $product->checksums[$api_details['id']] = $api_data_checksum;

                if (!empty($api_data[$csvcol['prodname']])) $product->name = trim($api_data[$csvcol['prodname']]);

                if (!empty($api_data[$csvcol['salepricenett']])) $product->setPrice(trim($api_data[$csvcol['salepricenett']]));
                $product->tax_rate=23;

                //if (!empty($api_data[$csvcol['long_description']])) $product->content['main'] = trim($api_data[$csvcol['long_description']]);

                // categories (add shop and bikes to all)
                /*
                $add_categories=array('1612759889529','1612761066456');
                if ($api_cat_name_to_cms_cat_ids){
                    if (!empty($api_data[$csvcol['main_category']])){
                        $api_cat_name=trim(strtolower($api_data[$csvcol['main_category']]));
                        if (array_key_exists($api_cat_name,$api_cat_name_to_cms_cat_ids)){
                            foreach($api_cat_name_to_cms_cat_ids[$api_cat_name] as $cat_id){
                                if (!in_array($cat_id,$add_categories))
                                    $add_categories[]=$cat_id;
                            }
                         }
                    }
                }

               if ($add_categories){

                    $old_categories = array();
                    if (!empty($product->category)){

                        $old_categories=$product->category;

                        // when category removed from product we need to resort priorities for that category
                        foreach ($old_categories as $category_id => $value){
                            if (!in_array($category_id,$add_categories)){
                                unset($product->category[$category_id]);
                                $update_categories[$category_id]=1;
                            }
                        }

                    }

                    foreach ($add_categories as $category_id){

                            if (empty($category_id)) continue;

                            // product already had this category
                            if (!empty($product->category[$category_id]))
                                continue;

                            // get the highest priority value in this category - this is a big slow down when many different categories are in the csv
                            if (!isset($priority[$category_id])){
                                $priority[$category_id]=0;
                                if ($items=get_items('products',$category_id,null,'priority')){
                                    $item_count=count($items);
                                    for ($item_num=0; $item_num<$item_count; $item_num++){
                                        if ($items[$item_num]['priority']>=$priority[$category_id]){$priority[$category_id]=$items[$item_num]['priority']+1;}
                                    }
                                    unset($items);
                                }
                                // $update_categories[$category_id]=1; // don't need to resort when adding, only if categories removed
                            }
                            $product->category[$category_id]['priority']=$priority[$category_id];
                            $priority[$category_id]++;

                    }


                }
                */

                // check for custom attributes that are currently setup
                if ($custom_attributes=get_custom_attributes('product')){

                    // $product->attr=array(); // debug temp reset measure

                    foreach($custom_attributes as $attr_id => $value){
                        if ($attr_id=='brand' && !empty($api_data[$csvcol['brand']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['brand']]);
                        elseif ($attr_id=='model' && !empty($api_data[$csvcol['model']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['model']]);
                        elseif ($attr_id=='size' && !empty($api_data[$csvcol['size']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['size']]);
                        elseif ($attr_id=='colour' && !empty($api_data[$csvcol['color']])){
                            $product->attr[$attr_id][0] = trim($api_data[$csvcol['color']]);
                            $product->attr[$attr_id]['color1']=guessColorCode($product->attr[$attr_id][0]);
                        }
                        elseif ($attr_id=='frame' && !empty($api_data[$csvcol['frame']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['frame']]);
                        elseif ($attr_id=='fork' && !empty($api_data[$csvcol['fork']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['fork']]);
                        elseif ($attr_id=='headset' && !empty($api_data[$csvcol['head_set']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['head_set']]);
                        elseif ($attr_id=='stem' && !empty($api_data[$csvcol['stem']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['stem']]);
                        elseif ($attr_id=='handle_bars' && !empty($api_data[$csvcol['handle_bars']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['handle_bars']]);
                        elseif ($attr_id=='tape_grips' && !empty($api_data[$csvcol['tape_grips']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['tape_grips']]);
                        elseif ($attr_id=='brakes' && !empty($api_data[$csvcol['brakes']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['brakes']]);
                        elseif ($attr_id=='tyres' && !empty($api_data[$csvcol['tyres']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['tyres']]);
                        elseif ($attr_id=='front_derailleur' && !empty($api_data[$csvcol['front_derailleur']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['front_derailleur']]);
                        elseif ($attr_id=='rear_derailleur' && !empty($api_data[$csvcol['rear_derailleur']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['rear_derailleur']]);
                        elseif ($attr_id=='shift_levers' && !empty($api_data[$csvcol['shift_levers']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['shift_levers']]);
                        elseif ($attr_id=='cassette' && !empty($api_data[$csvcol['cassette']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['cassette']]);
                        elseif ($attr_id=='crankset' && !empty($api_data[$csvcol['crankset']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['crankset']]);
                        elseif ($attr_id=='bottom_bracket' && !empty($api_data[$csvcol['bottom_bracket']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['bottom_bracket']]);
                        elseif ($attr_id=='pedals' && !empty($api_data[$csvcol['pedals']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['pedals']]);
                        elseif ($attr_id=='wheels' && !empty($api_data[$csvcol['wheels']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['wheels']]);
                        elseif ($attr_id=='front_tyre' && !empty($api_data[$csvcol['front_tyre']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['front_tyre']]);
                        elseif ($attr_id=='rear_tyre' && !empty($api_data[$csvcol['rear_tyre']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['rear_tyre']]);
                        elseif ($attr_id=='saddle' && !empty($api_data[$csvcol['saddle']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['saddle']]);
                        elseif ($attr_id=='seat_post' && !empty($api_data[$csvcol['seat_post']])) $product->attr[$attr_id][0] = trim($api_data[$csvcol['seat_post']]);


                    }
                }

                //$product->template = 'product';
                //$product->behaviour = 'Info Only';

                // image import
                // WARNING: importing lots of images can take a long time and a lot of disk space, make sure timeoouts are set long enough and there is enough space
                // where possible resize images down at the source before importing or use createthumb to store a smaller image, createthumb on every image will slow down the import proces further
                if ($rvi_import_images>0){
                    if (!empty($api_data[$csvcol['image']])){

                        $copy_img_url = trim($api_data[$csvcol['image']]); // make sure to trim url

                        if (strpos($copy_img_url,'http')!==false && strlen($copy_img_url)>7){

                            // get the image filename
                            $img_filename=basename($copy_img_url);
                            // the query ?wid=2000 hei=2000 adds a lot of whitespace to image size so remove them and use ?wid=640
                            if (strpos($img_filename,'?')!==false){
                                $filename_pieces = explode("?", basename($img_filename));
                                $img_filename = $filename_pieces[0];
                                $query = $filename_pieces[1];
                                $query_len=strlen($query);
                                $url_len = strlen($copy_img_url)-$query_len;
                                $copy_img_url = substr($copy_img_url,0,$url_len).'wid=1400';

                            }
                            // add .webp file extension
                            //$img_filename.='.webp';

                            // only import the image if filename does not already exist in product data
                            // some api it will not be possible to get the full name with extension before checking mime type
                            // this speeds up import and allows to batch in images over multiple imports
                                // a remote image can change but still use the same url/filename so really need to always import the image anyway? headers not always present/reliable?
                            if ($img_filename){

                                // ensure item dir exists so we can save the image files to it
                                $product->makeItemDir();

                                // create a checksum for image and only import the image if new or changed
                                // checksum should at least contain the image url
                                // to check if images chaged under same url we need to include headers that exist in the checksum Content-Type, Content-Length, Last-Modified, ETag
                                // for fastest performance exclude headers check and only use the image url
                                $skip_image_import = false;
                                $img_checksum = $copy_img_url;

                                // optionally comment out this if/else headers section for faster performance that ignores header changes
                                // when not using headers to update an existing image the image url should be changed for example with a query string
                                    if ($check_img_headers && $skip_image_import==false){
                                    if ($headers=get_http_headers($copy_img_url,1)){
                                        if (!empty($headers['etag'][0])) $img_checksum.=$headers['etag'][0];
                                        if (!empty($headers['last-modified'][0])) $img_checksum.=$headers['last-modified'][0];
                                        // content-type header must be an image type to import
                                        if (!empty($headers['content-type'][0])){
                                            $img_checksum.=$headers['content-type'][0];
                                            if (strpos($headers['content-type'][0],'image')===false) $skip_image_import = true;
                                        }
                                        else
                                            $skip_image_import = true;
                                        // content-length header must always be present to check file size before importing
                                        if (!empty($headers['content-length'][0])){
                                            $img_checksum.=$headers['content-length'][0];
                                            // skip really big images over 5mb
                                            if ($headers['content-length'][0]>5000000) $skip_image_import = true;
                                        }
                                        else
                                            $skip_image_import = true;
                                    }
                                    else
                                        $skip_image_import = true;

                                    if ($skip_image_import) $import_img_dl_fail++;
                                }

                                if ($skip_image_import!=true){
                                    $img_checksum = crc32($img_checksum);
                                    // check if item already has an image with this checksum
                                    if (!empty($product->image)){
                                        foreach ($product->image as $p_img_filename => $value){
                                            if (!empty($value['checksum']['api_'.$api_details['id']]) && $value['checksum']['api_'.$api_details['id']]==$img_checksum){
                                                $skip_image_import = true;
                                                break;
                                            }

                                        }

                                    }
                                }

                                if ($skip_image_import!=true){
                                    // import remote image file, checks mime type and returns appropriate file extension eg. .gif/.png/.webp/.jpg
                                    if ($img_type=api_import_image_file_curl($copy_img_url,$api_temp_fn)){

                                        $img_filename = $img_filename.$img_type;

                                        // rename the image to item directory and add image data to the item
                                        // we might consider createthumb to ensure image is not too big but this will slow down import
                                        if (rename($api_temp_fn,$product->itemDir().'/'.$img_filename)){
                                            $product->image[$img_filename]['alt_tag']=$product->name;
                                            // we add a cheksum so we don't have to download same image again unless it has changed
                                            if ($img_checksum) $product->image[$img_filename]['checksum']['api_'.$api_details['id']]=$img_checksum;
                                            $import_img_count++;
                                        }

                                    }
                                    else
                                        $import_img_dl_fail++;
                                }


                            }


                        }
                    }

                }


                if ($product->saveItem())
                    $updated_item_count++;

                // import only one item for testing
                if ($rvi_import_limit>0 && $updated_item_count>=$rvi_import_limit) break;

            }

            // import will finish up if it has been running too long, 900 secs = 15min
            $elapsed_time = time()-CMS_INIT_TIME;
            if ($elapsed_time > $process_time_limit){
                $import_timeout = $elapsed_time;
                break;
            }

        }

    }

    // re-sort priorities for categories that product is no longer in
    if (!empty($update_categories)){
        update_api_import_log($api_details['id'],"API ".$api_details['id']." data import sorting category priorities on ".count($update_categories)." categories..");
        foreach ($update_categories as $category_id => $value){
            sort_category_priorities('products',$category_id);
        }
        unset($update_categories);
    }

    if ($updated_item_count)
        rebuildItemIndex('products');

    if (isset($unlock_key))
        unlock_cms($unlock_key);

    if (isset($import_timeout))
        update_api_import_log($api_details['id'],"Warning: API ".$api_details['id']." data import timeout - ".$updated_item_count."/".$api_item_count." products updated, ".$overwrite_item_count." overwritten, ".$import_img_count." images imported, ".$import_img_dl_fail." downloads failed in ".(time()-CMS_INIT_TIME)." seconds");
    else
    {
        if ($rvi_import_limit == 1 && $updated_item_count>0)
            update_api_import_log($api_details['id'],"API ".$api_details['id']." data import complete - product ".$product->id." updated - <a href=\"".$product->getURL()."\" target=\"_blank\">view</a> / <a href=\"?editProduct=".$product->id."\">edit</a>");
        else{
            if ($import_img_dl_fail>0)
                update_api_import_log($api_details['id'],"Warning: API ".$api_details['id']." data import complete - ".$updated_item_count."/".$api_item_count." products updated, ".$overwrite_item_count." overwritten, ".$import_img_count." images imported, ".$import_img_dl_fail." downloads failed in ".(time()-CMS_INIT_TIME)." seconds");
            else
                update_api_import_log($api_details['id'],"API ".$api_details['id']." data import complete - ".$updated_item_count."/".$api_item_count." products updated, ".$overwrite_item_count." overwritten, ".$import_img_count." images imported, ".$import_img_dl_fail." downloads failed in ".(time()-CMS_INIT_TIME)." seconds");
        }
    }


?>
