<?php

    /*
        Description: display linked category images, can be used as gallery or to display brand images etc.
    */

    // each category content template file must have a unique namespace
    // the namespace must be category\[content template filename] all lower-case using backslash to seperate and without file extension
    // this allows the CMS to use multiple content templates that use the same function names
    namespace category\ct_catimage_gallery;

    // this file is used to format the display of a category page and it's item listings (if any)
    // multiple layouts can be achieved by using the layout parameter eg. a grid and list view on the same page or recently viewed products

    // this function is used to display the category content and any listing items
    // $cat is an array that contains generated output data and content; total_item_count, start_item, items_per_page, page_num, items_on_page_count, page_nav
    // $cat['items_content'][] (an array with each key containing html output generated with the item_output function below eg. $cat[items_output]['main'])
    function category_output($category,$search,$cat=array(),$layout='main'){

        $category_output="";

        if ($category->id=='2' && $cat['total_item_count']<1) return;

        if ($layout=='main'){

            $category_output="<div class=\"cms-content cms-category\">"; // open cms-content div

            // show search text as page heading if category is displaying search results
            if (!empty($search)){
                    $category_output.="<h1>Search Results</h1>";
                    if ($cat['total_item_count']<1)
                        $category_output.="<p>No products found.</p>";
                    else{
                        $category_output.="<p>".$cat['total_item_count']." products found.</p>";
                        $category_output.="<div class=\"cms-product-listings\">".$cat['items_content']['main']."</div>";
                        $category_output.=$cat['page_nav']; // pagination
                    }
            }
            else{

                    // display category name as page heading
                    if ($category->heading_visible>0 && $category->name!="")
                        $category_output.="<h1>".htmlspecialchars($category->name)."</h1>";
                    // display category content
                    if ($category->description_visible>0 && $category->content['main']!="")
                        $category_output.="<div class=\"cms-content-main\">".$category->content['main']."</div>";

                    if ($category->image){

                        $img_gallery="";
                        foreach ($category->image as $image_filename => $img_details){

                            if (empty($img_details['thumbnail'])) break;

                            // image and name beneath
                            $img_g="<div style=\"width: 100%;height: 280px; padding: 0px;margin-bottom: 15px\"><img src=\"".cms_root_url.'sys/images/'.$img_details['thumbnail']."\" alt=\"".$img_details['alt_tag']."\" style=\"width: 100%; height: 100%; object-fit: cover\"></div>";
                            $img_g.="<h3 style=\"color: #009fff; line-height:1.4; font-weight: normal\">".ucwords(strtolower($img_details['alt_tag']))."</h3>";

                            // wrap with link
                            if (!empty($img_details['link'])) $img_g="<a href=\"".$img_details['link']."\">".$img_g."</a>";

                            // wrap with container
                            $img_gallery.="<div style=\"display: flex; flex-direction: column;flex: 1 1 280px; max-width: 280px; line-height: 1em; text-align: center; margin: 0.5em\">".$img_g."</div>\n";  // single category image link container


                        }
                        if ($img_gallery) $category_output.="<div style=\"display: flex; flex-flow: row wrap; justify-content: space-around;padding: 20px 0 0 0;margin: 20px 0\">".$img_gallery."</div>";

                    }

            }

            $category_output.="</div>"; // close cms-content div
        }
        elseif(isset($category->content[$layout])){
            // we just output other content areas as-is
            $category_output=$category->content[$layout];
        }

        return $category_output;
    }

    // this function is used to display a single item listing
    // it is iterated through for each item listing displayed on the category page
    // the resulting html is then passed into the category_ouput function above as $cat['items_content']['main']
    // additional output keys can be used if needed eg. $cat['items_content']['my_other_key']
    function item_output($category,$item,$item_num,$layout=null){

         $item_output['main']="";

        return $item_output;

    }


?>