<?php

    // each category content template file must have a unique namespace
    // the namespace must be category\[content template filename] all lower-case using backslash to seperate and without file extension
    // this allows the CMS to use multiple content templates that use the same function names
    namespace category\ct_default;

    // this content template file is used to format the display of category page content and it's item listings (if any)
    // this content template can deliver multiple layouts, achieved using the layout parameter which typically contains the content area name (eg. 'main' or a custom content area name)
    // a custom layout name can also be set when calling displayCategory() function in a custom include file eg. to output a grid and list view on the same page or recently viewed products

    // settings for this content template
    function ct_settings(){
        $ct_settings=array(
            'number_of_page_links'  => 5,   // the max number of page links to be shown either side of current page nav link
            'tnWidth'               => 200, // image thumbnail display width
            'tnHeight'              => 200  // image thumbnail display height
        );
        return $ct_settings;
    }

    // this function is used to display the category content and any listing items
    // $cat is an array that contains generated output data and content; total_item_count, start_item, items_per_page, page_num, items_on_page_count, page_nav
    // $cat['items_content'][] (an array with each key containing html output generated with the item_output function below eg. $cat[items_output]['main'])
    function category_output($category,$search,$cat=array(),$layout='main'){

        $category_output="";

        if ($layout=='main'){

            $category_output="<div class=\"cms-content cms-category\">"; // open cms-content div

            // display category name as page heading
            if ($category->name && $category->heading_visible)
                $category_output.="<h1>".htmlspecialchars($category->name)."</h1>";

            // display category content
            if (!empty($category->content['main']) && $category->description_visible)
                $category_output.="<div class=\"cms-content-main\">".$category->content['main']."</div>";

            $category_output.="</div>"; // close cms-content div
        }
        elseif(isset($category->content[$layout])){
            // we just output other content areas as-is
            $category_output=$category->content[$layout];
        }

        return $category_output;
    }

    // this function is used to display a single item listing
    // it is iterated through for each item listing displayed on the category page
    // the resulting html is then passed into the category_ouput function above as $cat['items_content']['main']
    // additional output keys can be used if needed eg. $cat['items_content']['my_other_key']
    function item_output($category,$item,$item_num,$layout=null){

        $ct_settings=ct_settings();

        $item_output['main']="";

        // default content template does not show items of any type
        return $item_output;

    }


?>