<?php

    // each category content template file must have a unique namespace
    // the namespace must be category\[content template filename] all lower-case using backslash to seperate and without file extension
    // this allows the CMS to use multiple content templates that use the same function names
    namespace category\ct_news_default;

    // this file is used to format the display of a category page and it's item listings (if any)
    // multiple layouts can be achieved by using the layout parameter eg. a grid and list view on the same page or recently viewed products

    // settings for this content template
    function ct_settings(){
        $ct_settings=array(
            'number_of_page_links'  => 5,   // the max number of page links to be shown either side of current page nav link
            'tnWidth'               => 220, // image thumbnail display width
            'tnHeight'              => 200  // image thumbnail display height
        );
        return $ct_settings;
    }

    // this function is used to display the category content and any listing items
    // $cat is an array that contains generated output data and content; total_item_count, start_item, items_per_page, page_num, items_on_page_count, page_nav
    // $cat['items_content'][] (an array with each key containing html output generated with the item_output function below eg. $cat['items_content']['main'])
    function category_output($category,$search,$cat=array(),$layout='main'){

        $category_output="";

        if ($layout=='main'){

            $category_output="<div class=\"cms-content cms-category\">"; // open cms-content div

            // show search text as page heading if category is displaying search results
            if (!empty($search)){
                $category_output.="<h1>Search Results</h1>";
                if ($cat['total_item_count']<1){$category_output.="<p>No articles found.</p>";}
                else{
                    $category_output.="<p>".$cat['total_item_count']." articles found.</p>";
                    $category_output.="<div class=\"cms-news-listings\">".$cat['items_content']['main']."</div>";
                    $category_output.=$cat['page_nav']; // pagination
                }
            }
            else{

                // display category name as page heading
                if ($category->heading_visible && $category->name)
                    $category_output.="<h1>".htmlspecialchars($category->name)."</h1>";

                // display category content
                if ($category->description_visible && !empty($category->content['main']))
                    $category_output.="<div class=\"cms-content-main\">".$category->content['main']."</div>";

                if ($category->listings_visible>0){
                    if (empty($cat['items_content']['main']))
                        $category_output.="<p>No articles found.</p>";
                    else{
                        $category_output.="<div class=\"cms-news-listings\">".$cat['items_content']['main']."</div>";
                        $category_output.=$cat['page_nav']; // pagination
                    }
                }

            }

            $category_output.="</div>"; // close cms-content div
        }
        elseif(isset($category->content[$layout])){
            // we just output other content areas as-is
            $category_output=$category->content[$layout];
        }

        return $category_output;
    }

    // this function is used to display a single item listing
    // it is iterated through for each item listing displayed on the category page
    // the resulting html is then passed into the category_ouput function above as $cat['items_content']['main']
    // additional output keys can be used if needed eg. $cat['items_content']['my_other_key']
    function item_output($category,$item,$item_num,$layout=null){

        global $txt; // language strings

        $ct_settings=ct_settings();

        // set variables on first item for all items
        // 90% width area divided by number of cols
        $listing_width=90/$category->cols;

        $outer_border=1;
        $inner_border=5;

        $item_output['main']="<div class=\"news_listing\">";

        // show thumbnail image or no-image
        if (!empty($item->thumbnail[0])){
            $item_output['main'].="<div class=\"img_cell\" style=\"float: left; max-width: ".($ct_settings['tnWidth']+($inner_border*2))."px; border: #b2b2b2 solid ".$outer_border."px; margin: 0 15px 10px 0\"><div style=\"border: #ffffff solid ".$inner_border."px;\"><a href=\"".$item->getURL()."\"><img src=\"".cms_image_url.htmlspecialchars($item->thumbnail[0])."\" alt=\"".htmlspecialchars($item->name)."\" style=\"oject-fit: cover\" /></a></div></div>";
            //$item_output['main'].="<div class=\"img_cell\" style=\"flex: 1 1 ".$ct_settings['tnWidth']."px\"><div style=\"height: ".$ct_settings['tnHeight']."px\"><a href=\"".$item->getURL()."\"><img src=\"".cms_image_url.htmlspecialchars($item->thumbnail[0])."\" alt=\"".htmlspecialchars($item->name)."\" /></a></div></div>";
        }

        $item_output['main'].="<div class=\"news_listing_name\"><a href=\"".$item->getURL()."\">".htmlspecialchars($item->name)."</a></div>";
        if ($item->content['main']){$item_output['main'].="<div class=\"news_listing_description\">".substr(strip_tags($item->content['main']),0,250)."... <a href=\"".$item->getURL()."\">read more</a></div>";}

        // get article owner name
        $article_owner='Admin';
        if ($item->owner){
            $user = new User();
            if ($user->getData($item->owner))
                $article_owner=$user->name;
        }


        // calculate number of days since update
        /*
        // not using because of caching inaccurate today/yesterday text
        $modified_time_midnight=mktime(0, 0, 0, date('m',$item->date), date('d',$item->date), date('Y',$item->date));
        $num_of_days=(time()-$modified_time_midnight)/86400;
        if ($num_of_days<1){$post_time='Today';}
        elseif ($num_of_days<2){$post_time='Yesterday';}
        else {$post_time=date('jS F Y',$item->date);}
        $item_output['main'].="<div class=\"news_listing_date\">".$post_time." by ".$article_owner."</div>";
        */
        $item_output['main'].="<div class=\"news_listing_date\">".date('jS F Y',$item->date)." by ".$article_owner."</div>";

        $item_output['main'].="<div style=\"clear: both\"></div></div>";

        return $item_output;

    }


?>