<?php

    /*
        Description: default content template for displaying products responsively
    */

    // each category content template file must have a unique namespace
    // the namespace must be category\[content template filename] all lower-case using backslash to seperate and without file extension
    // this allows the CMS to use multiple content templates that use the same function names
    namespace category\ct_products_default;

    // this file is used to format the display of a category page and it's item listings (if any)
    // multiple layouts can be achieved by using the layout parameter eg. a grid and list view on the same page or recently viewed products

    // settings for this content template
    function ct_settings(){
        $ct_settings=array(
            'number_of_page_links'  => 5,   // the max number of page links to be shown either side of current page nav link
            'tnWidth'               => 200, // image thumbnail display width
            'tnHeight'              => 200  // image thumbnail display height
        );
        return $ct_settings;
    }

    // this function is used to display the category content and any listing items
    // $cat is an array that contains generated output data and content; total_item_count, start_item, items_per_page, page_num, items_on_page_count, page_nav
    // $cat['items_content'][] (an array with each key containing html output generated with the item_output function below eg. $cat['items_content']['main'])
    function category_output($category,$search,$cat=array(),$layout='main'){

        $category_output="";

        if ($layout=='main'){

            $category_output="<div class=\"cms-content cms-category\">"; // open cms-content div

            // show search text as page heading if category is displaying search results
            if (!empty($search)){
                    $category_output.="<h1>Search Results</h1>";
                    if ($cat['total_item_count']<1){$category_output.="<p>No products found.</p>";}
                    else{
                        $category_output.="<p>".$cat['total_item_count']." products found.</p>";
                        $category_output.="<div class=\"cms-product-listings\">".$cat['items_content']['main']."</div>";
                        $category_output.=$cat['page_nav']; // pagination
                    }
            }
            else{

                    // display category name as page heading
                    if ($category->heading_visible && $category->name!="")
                        $category_output.="<h1>".htmlspecialchars($category->name)."</h1>";

                    // display category content
                    if ($category->description_visible && !empty($category->content['main']))
                        $category_output.="<div class=\"cms-content-main\">".$category->content['main']."</div>";

                    if ($category->listings_visible>0){
                        if (empty($cat['items_content']['main']))
                            $category_output.="<p>No products found.</p>";
                        else{
                            $category_output.="<div class=\"cms-product-listings\">".$cat['items_content']['main']."</div>";
                            $category_output.=$cat['page_nav']; // pagination
                        }
                    }
            }

            $category_output.="</div>"; // close cms-content div

        }
        elseif(isset($category->content[$layout])){
            // we just output other content areas as-is
            $category_output=$category->content[$layout];
        }

        return $category_output;
    }

    // this function is used to display a single item listing
    // it is iterated through for each item listing displayed on the category page
    // the resulting html is then passed into the category_ouput function above as $cat['items_content']['main']
    // additional output keys can be used if needed eg. $cat['items_content']['my_other_key']
    function item_output($category,$item,$item_num,$layout=null){

        global $txt; // language strings

        $ct_settings=ct_settings();

        // set variables on first item for all items
        // 90% width area divided by number of cols
        $listing_width=90/$category->cols;

        $item_output['main']="<div class=\"cms-product-listing\" style=\"flex: 1 1 ".$listing_width."%\">";

        // show thumbnail image or no-image
        if (empty($item->thumbnail[0])){$item_output['main'].="<div class=\"img_cell\" style=\"flex: 1 1 ".$ct_settings['tnWidth']."px\"><div style=\"height: ".$ct_settings['tnHeight']."px\">No Image</div></div>";}
        else{$item_output['main'].="<div class=\"img_cell\" style=\"flex: 1 1 ".$ct_settings['tnWidth']."px\"><div style=\"height: ".$ct_settings['tnHeight']."px\"><a href=\"".$item->getURL()."\"><img src=\"".cms_image_url.htmlspecialchars($item->thumbnail[0])."\" alt=\"".htmlspecialchars($item->name)."\" /></a></div></div>";}

        $item_output['main'].="<div class=\"cms-product-listing-detail\"><div class=\"product_name\"><a href=\"".$item->getURL()."\">".htmlspecialchars($item->name)."</a></div>";
        if (!empty($item->code)){$item_output['main'].="<div class=\"product_code\">".$item->code."</div>";}
        if (!empty($item->short_description)){$item_output['main'].="<div class=\"product_description\">".htmlspecialchars($item->short_description)."</div>";}
        $display_price=$item->displayPrice();
        if (!empty($display_price)){$item_output['main'].="<div class=\"product_price\">".$display_price."</div>";}
        $item_form="";
        if (!empty($item->display_option)){$item_form.="<div>".$item->display_option."</div>";}
        /*
        if ($cmsSettings->shop_enabled && $item->behaviour=='Add to Cart'){
        if ($item_details['note_form_enabled']>0 || $item_details['file_upload_enabled']>0){
        $item_output['main'].="<div class=\"product_qty\"><b>Qty:</b> <input type=\"text\" name=\"qty\" value=\"1\" /></div>
                        <div class=\"product_buy\"><input type=\"image\" src=\"".theme_file_url('images/shop/add_to_cart.png',display_theme)."\" style=\"padding-top: 5px\" /><input type=\"hidden\" name=\"product\" value=\"".$item->id."\" /></div>";
        }
        else{
        $item_output['main'].="<div class=\"product_qty\"><b>Qty:</b> <input type=\"text\" name=\"qty\" value=\"1\" /></div>
                        <div class=\"product_buy\"><input type=\"image\" src=\"".theme_file_url('images/shop/add_to_cart.png',display_theme)."\" style=\"padding-top: 5px\" /><input type=\"hidden\" name=\"add_to_cart\" value=\"".$item->id."\" /></div>";
        }
        }
        elseif(substr($item->behaviour,0,7)=='Listing'){
                $item_output['main'].="<div class=\"product_buy\"><input type=\"image\" src=\"".theme_file_url('images/shop/continue.png',display_theme)."\" style=\"padding-top: 5px\" /><input type=\"hidden\" name=\"product\" value=\"".$item->id."\" /></div>";
        }
        */
        if ($item->behaviour=='Add to Cart'){
            /*
            product status values
            0 = out of stock (auto)
            1 = in stock (auto+visible)
            2 = out of stock (visible)
            3 = out of stock (hidden)
            4 = in stock (hidden)
            */
            if ($item->status==1){
                if ($item->note_form_enabled>0 || $item->file_upload_enabled>0){
                    $item_form.="<div class=\"product_qty\">Qty: <input type=\"text\" id=\"qty\" name=\"qty\" value=\"1\" /></div><div class=\"product_buy\"><a href=\"".$item->getURL()."\" class=\"cms_btn cms_btn_info\">".$txt['btn_more_info']."</a><button type=\"submit\" name=\"product\" value=\"".$item->id."\" class=\"cms_btn cms_btn_add_to_cart\">".$txt['btn_add_to_cart']."</button></div>";
                }
                else{
                    $item_form.="<div class=\"product_qty\">Qty: <input type=\"text\" id=\"qty\" name=\"qty\" value=\"1\" /></div><div class=\"product_buy\"><a href=\"".$item->getURL()."\" class=\"cms_btn cms_btn_info\">".$txt['btn_more_info']."</a><button type=\"submit\" name=\"add_to_cart\" value=\"".$item->id."\" class=\"cms_btn cms_btn_add_to_cart\">".$txt['btn_add_to_cart']."</button></div>";
                }
            }
            else{
                $item_form.="<div class=\"product_buy\"><a href=\"".$item->getURL()."\" class=\"cms_btn cms_btn_info\">".$txt['btn_more_info']."</a> Out of Stock</div>";
            }
        }
        elseif ($item->behaviour=='Info Only'){
            $item_form.="<div class=\"product_buy\"><button type=\"submit\" name=\"product\" value=\"".$item->id."\" class=\"cms_btn cms_btn_info\">".$txt['btn_more_info']."</button></div>";
        }
        elseif(substr($item->behaviour,0,7)=='Listing' || substr($item->behaviour,0,7)=='Application'){
            /*$item_form="<div class=\"product_buy\"><input type=\"image\" src=\"".theme_file_url('images/shop/add_to_cart.png',display_theme)."\" alt=\"Add to Cart\" style=\"padding-top: 5px\" /><input type=\"hidden\" name=\"product\" value=\"".$item->id."\" /></div>";*/
            $item_form.="<div class=\"product_buy\"><button type=\"submit\" class=\"cms_btn cms_btn_add_to_cart\">".$txt['btn_add_to_cart']."</button><input type=\"hidden\" name=\"product\" value=\"".$item->id."\" /></div>";
        }
        if (!empty($item_form)){$item_output['main'].="\n<form method=\"post\" class=\"cms_product_form\">".$item_form."</form>";}

        // close container div for product_listing_detail
        $item_output['main'].="</div>";

        // close container div for this item
        $item_output['main'].="</div>";

        return $item_output;

    }


?>